/**
 * Being.java
 * part of cellularAutomata
 *
 * written by Matthew Shanley
 * matthewshanley@littlesecretsrecords.com
 */

package com.littlesecretsrecords.cellularautomata;

import java.lang.*;

public class Being {
    private ThoughtWord myThoughtWord;   // ThoughtWord if carrying one
    private String role;   // possible values: collector, disperser
    private World myWorld;   // World in which I live
    private int xPos;   // x position in world
    private int yPos;   // y position in world
    private int lastXmove;   // how much I moved in X direction last time (-1, 0, 1)
    private int lastYmove;   // how much I moved in Y direction last time (-1, 0 1)
    private int movesSincePickup;   // how many moves I've made since picking up my ThoughtWord. The longer it's been, the more likely I am to drop it in an acceptable spot.

    // Constructor
    public Being(World initWorld) {
	if (Math.random() < 0.5) {   // 50% chance
	    this.role = "collector";
	} else {
	    this.role = "disperser";
	}

	this.myWorld = initWorld;

	// Randomly position self within world
	this.xPos = (int)(Math.random()*myWorld.getWidth());
	this.yPos = (int)(Math.random()*myWorld.getHeight());

	this.lastXmove = 0;
	this.lastYmove = 0;
    }
    
    public int getXpos() {
	return xPos;
    }

    public int getYpos() {
	return yPos;
    }

    // Public Methods
    // Decide whether we should take or leave a ThoughtWord
    // and if so, do so
    public void takeLeaveThoughtWord() {
	/*
	 * If I don't have a ThoughtWord,
	 * see if there is at least one on the Site I am on.
	 * If so, pick one up
	 */
	if (myThoughtWord == null) {
	    if (myWorld.getSite(xPos, yPos).getNumThoughtWords() > 0) {
		myThoughtWord = myWorld.getSite(xPos, yPos).takeThoughtWord();
		movesSincePickup = 0;
	    }
	} else {
	    /*
	     * I do have a ThoughtWord, so
	     * test if this is an appropriate spot to leave ThoughtWord:
	     * If I'm a collector, I only leave on spots 
	     * that already have at least one ThoughtWord.
	     * If I'm a disperser, I only leave on spots
	     * that have no ThoughtWords.
	     * If this is an appropriate spot, determine whether to leave 
	     * my ThoughtWord based on movesSincePickup
	     */
	    if (Math.random() < (movesSincePickup*0.05)) {
		if ( ((role=="collector") && (myWorld.getSite(xPos, yPos).getNumThoughtWords()>0)) ||  ((role=="disperser") && (myWorld.getSite(xPos, yPos).getNumThoughtWords()==0)) ){
		    myWorld.getSite(xPos, yPos).leaveThoughtWord(myThoughtWord);
		    myThoughtWord = null;
		}
	    }
	}
    }

    // Move (weighted randomly)
    public void move() {
	double tempRand;
	float xGradient = myWorld.getXgradient();
	float yGradient = myWorld.getYgradient();

	// Determine how to move in X
	tempRand = Math.random();
	if (lastXmove < 0) {
	    if (tempRand < 0.5+xGradient) {
		if (xPos==0) { xPos = myWorld.getWidth()-1; }
		else { xPos--; }
		lastXmove = -1;
	    }
	    else if (tempRand > 0.75+xGradient) { 
		if (xPos==(myWorld.getWidth()-1)) { xPos = 0; }
		else { xPos++; }
		lastXmove = 1;
	    } else {
		lastXmove = 0;
	    }
	} else if (lastXmove > 0) {
	    if (tempRand < 0.25+xGradient) {
		if (xPos==0) { xPos = myWorld.getWidth()-1; }
		else { xPos--; }
		lastXmove = -1;
	    }
	    else if (tempRand > 0.5+xGradient) {
		if (xPos==(myWorld.getWidth()-1)) { xPos = 0; }
		else { xPos++; }
		lastXmove = 1;
	    } else {
		lastXmove = 0;
	    }
	} else {
	    if (tempRand < 0.33+xGradient) {
		if (xPos==0) { xPos = myWorld.getWidth()-1; }
		else { xPos--; }
		lastXmove = -1;
	    }
	    else if (tempRand > 0.67+xGradient) {
		if (xPos==(myWorld.getWidth()-1)) { xPos = 0; }
		else { xPos++; }
		lastXmove = 1;
	    } else {
		lastXmove = 0;
	    }
	}

	// Determine how to move in Y
	tempRand = Math.random();
	if (lastYmove < 0) {
	    if (tempRand < 0.5+yGradient) {
		if (yPos==0) { yPos = myWorld.getHeight()-1; }
		else { yPos--; }
		lastYmove = -1;
	    }
	    else if (tempRand > 0.75+ yGradient) {
		if (yPos==(myWorld.getHeight()-1)) { yPos = 0; }
		else { yPos++; }
		lastYmove = 1;
	    } else {
		lastYmove = 0;
	    }
	} else if (tempRand > 0) {
	    if (tempRand < 0.25+yGradient) {
		if (yPos==0) { yPos = myWorld.getHeight()-1; }
		else { yPos--; }
		lastYmove = -1;
	    }
	    else if (tempRand > 0.5+yGradient) {
		if (yPos==(myWorld.getHeight()-1)) { yPos = 0; }
		else { yPos++; }
		lastYmove = 1;
	    } else { 
		lastYmove = 0;
	    }
	} else {
	    if (tempRand < 0.33+yGradient) {
		if (yPos==0) { yPos = myWorld.getHeight()-1; }
		else { yPos--; }
		lastYmove = -1;
	    }
	    else if (tempRand > 0.67+yGradient) {
		if (yPos==(myWorld.getHeight()-1)) { yPos = 0; }
		else { yPos++; }
		lastYmove = 1;
	    } else {
		lastYmove = 0;
	    }
	}
	
	movesSincePickup++;
    }
}
